@@ -1,35 +1,33 @@
 """
-    pint.facets.plain.quantity
-    ~~~~~~~~~~~~~~~~~~~~~~~~~
+pint.facets.plain.quantity
+~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: 2022 by Pint Authors, see AUTHORS for more details.
-    :license: BSD, see LICENSE for more details.
+:copyright: 2022 by Pint Authors, see AUTHORS for more details.
+:license: BSD, see LICENSE for more details.
 """
 
 from __future__ import annotations
 
-
 import copy
 import datetime
 import locale
 import numbers
 import operator
+from collections.abc import Callable, Iterable, Iterator, Sequence
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
-    overload,
     Generic,
     TypeVar,
-    Optional,
-    Union,
+    overload,
 )
-from collections.abc import Iterator, Sequence
 
-from ..._typing import UnitLike, QuantityOrUnitLike, Magnitude, Scalar
+from ..._typing import Magnitude, QuantityOrUnitLike, Scalar, UnitLike
 from ...compat import (
     HAS_NUMPY,
+    Self,
     _to_magnitude,
+    deprecated,
     eq,
     is_duck_array_type,
     is_upcast_type,
@@ -44,8 +42,8 @@ from ...util import (
     logger,
     to_units_container,
 )
-from .definitions import UnitDefinition
 from . import qto
+from .definitions import UnitDefinition
 
 if TYPE_CHECKING:
     from ..context import Context
@@ -57,7 +55,7 @@ if TYPE_CHECKING:
 
 try:
     import uncertainties.unumpy as unp
-    from uncertainties import ufloat, UFloat
+    from uncertainties import UFloat, ufloat
 
     HAS_UNCERTAINTIES = True
 except ImportError:
@@ -136,15 +134,13 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
 
     """
 
-    #: Default formatting string.
-    default_format: str = ""
     _magnitude: MagnitudeT
 
     @property
     def ndim(self) -> int:
         if isinstance(self.magnitude, numbers.Number):
             return 0
-        if str(self.magnitude) == "<NA>":
+        if str(type(self.magnitude)) == "NAType":
             return 0
         return self.magnitude.ndim
 
@@ -169,27 +165,23 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
 
     @overload
     def __new__(
-        cls, value: MagnitudeT, units: Optional[UnitLike] = None
-    ) -> PlainQuantity[MagnitudeT]:
-        ...
+        cls, value: MagnitudeT, units: UnitLike | None = None
+    ) -> PlainQuantity[MagnitudeT]: ...
 
     @overload
     def __new__(
-        cls, value: str, units: Optional[UnitLike] = None
-    ) -> PlainQuantity[Any]:
-        ...
+        cls, value: str, units: UnitLike | None = None
+    ) -> PlainQuantity[Any]: ...
 
     @overload
     def __new__(  # type: ignore[misc]
-        cls, value: Sequence[ScalarT], units: Optional[UnitLike] = None
-    ) -> PlainQuantity[Any]:
-        ...
+        cls, value: Sequence[ScalarT], units: UnitLike | None = None
+    ) -> PlainQuantity[Any]: ...
 
     @overload
     def __new__(
-        cls, value: PlainQuantity[Any], units: Optional[UnitLike] = None
-    ) -> PlainQuantity[Any]:
-        ...
+        cls, value: PlainQuantity[Any], units: UnitLike | None = None
+    ) -> PlainQuantity[Any]: ...
 
     def __new__(cls, value, units=None):
         if is_upcast_type(type(value)):
@@ -262,8 +254,18 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
         )
         return ret
 
+    @deprecated(
+        "This function will be removed in future versions of pint.\n"
+        "Use ureg.formatter.format_quantity_babel"
+    )
+    def format_babel(self, spec: str = "", **kwspec: Any) -> str:
+        return self._REGISTRY.formatter.format_quantity_babel(self, spec, **kwspec)
+
+    def __format__(self, spec: str) -> str:
+        return self._REGISTRY.formatter.format_quantity(self, spec)
+
     def __str__(self) -> str:
-        return str(self.magnitude) + " " + str(self.units)
+        return self._REGISTRY.formatter.format_quantity(self)
 
     def __bytes__(self) -> bytes:
         return str(self).encode(locale.getpreferredencoding())
@@ -325,6 +327,10 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
         """ """
         return not bool(self.to_root_units()._units)
 
+    def unit_items(self) -> Iterable[tuple[str, Scalar]]:
+        """A view of the unit items."""
+        return self._units.unit_items()
+
     @property
     def dimensionless(self) -> bool:
         """ """
@@ -332,7 +338,7 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
 
         return not bool(tmp.dimensionality)
 
-    _dimensionality: Optional[UnitsContainerT] = None
+    _dimensionality: UnitsContainerT | None = None
 
     @property
     def dimensionality(self) -> UnitsContainerT:
@@ -414,6 +420,8 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
 
     @classmethod
     def from_tuple(cls, tup):
+        for units_tup in tup[1]:
+            cls._REGISTRY.get_name(units_tup[0])
         return cls(tup[0], cls._REGISTRY.UnitsContainer(tup[1]))
 
     def to_tuple(self) -> tuple[MagnitudeT, tuple[tuple[str, ...]]]:
@@ -427,7 +435,7 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
         return self._REGISTRY.get_compatible_units(self._units)
 
     def is_compatible_with(
-        self, other: Any, *contexts: Union[str, Context], **ctx_kwargs: Any
+        self, other: Any, *contexts: str | Context, **ctx_kwargs: Any
     ) -> bool:
         """check if the other object is compatible
 
@@ -469,7 +477,7 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
             with self._REGISTRY.context(*contexts, **ctx_kwargs):
                 return self._REGISTRY.convert(self._magnitude, self._units, other)
 
-        return self._REGISTRY.convert(self._magnitude, self._units, other)
+        return self._REGISTRY.convert(self._magnitude, self._units, other, **ctx_kwargs)
 
     def _convert_magnitude(self, other, *contexts, **ctx_kwargs):
         if contexts:
@@ -481,10 +489,11 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
             self._units,
             other,
             inplace=is_duck_array_type(type(self._magnitude)),
+            **ctx_kwargs,
         )
 
     def ito(
-        self, other: Optional[QuantityOrUnitLike] = None, *contexts, **ctx_kwargs
+        self, other: QuantityOrUnitLike | None = None, *contexts, **ctx_kwargs
     ) -> None:
         """Inplace rescale to different units.
 
@@ -506,8 +515,8 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
         return None
 
     def to(
-        self, other: Optional[QuantityOrUnitLike] = None, *contexts, **ctx_kwargs
-    ) -> PlainQuantity:
+        self, other: QuantityOrUnitLike | None = None, *contexts, **ctx_kwargs
+    ) -> Self:
         """Return PlainQuantity rescaled to different units.
 
         Parameters
@@ -575,6 +584,8 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
     ito_preferred = qto.ito_preferred
     to_reduced_units = qto.to_reduced_units
     ito_reduced_units = qto.ito_reduced_units
+    to_unprefixed = qto.to_unprefixed
+    ito_unprefixed = qto.ito_unprefixed
 
     # Mathematical operations
     def __int__(self) -> int:
@@ -739,21 +750,44 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
                 raise DimensionalityError(self._units, "dimensionless")
             return self.__class__(magnitude, units)
 
+        # Special case for logarithmic units: dB can be added to dBm, dBW, etc.
+        # Get non-multiplicative units before checking dimensionality
+        self_non_mul_units = self._get_non_multiplicative_units()
+        other_non_mul_units = other._get_non_multiplicative_units()
+
+        # Next we define some variables to make if-clauses more readable.
+        # We already have self_non_mul_units and other_non_mul_units from our logarithmic check
+        is_self_multiplicative = len(self_non_mul_units) == 0
+        is_other_multiplicative = len(other_non_mul_units) == 0
+
+        if len(self_non_mul_units) == 1:
+            self_non_mul_unit = self_non_mul_units[0]
+        if len(other_non_mul_units) == 1:
+            other_non_mul_unit = other_non_mul_units[0]
+
+        # Check if we're dealing with logarithmic units that can be added (dB + dBm, etc.)
+        if self._is_logarithmic and other._is_logarithmic:
+            self_base = self.to_base_units()
+            other_base = other.to_base_units()
+            if op == operator.add:
+                result = self_base * other_base
+            elif op == operator.sub:
+                result = self_base / other_base
+
+            if self_base.dimensionless and other_base.dimensionless:
+                return result.to(self._units)
+            elif self_base.dimensionless:
+                return result.to(other._units)
+            elif other_base.dimensionless:
+                return result.to(self._units)
+            else:
+                return result
+
         if not self.dimensionality == other.dimensionality:
             raise DimensionalityError(
                 self._units, other._units, self.dimensionality, other.dimensionality
             )
 
-        # Next we define some variables to make if-clauses more readable.
-        self_non_mul_units = self._get_non_multiplicative_units()
-        is_self_multiplicative = len(self_non_mul_units) == 0
-        if len(self_non_mul_units) == 1:
-            self_non_mul_unit = self_non_mul_units[0]
-        other_non_mul_units = other._get_non_multiplicative_units()
-        is_other_multiplicative = len(other_non_mul_units) == 0
-        if len(other_non_mul_units) == 1:
-            other_non_mul_unit = other_non_mul_units[0]
-
         # Presence of non-multiplicative units gives rise to several cases.
         if is_self_multiplicative and is_other_multiplicative:
             if self._units == other._units:
@@ -823,8 +857,7 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
         ...
 
     @overload
-    def __iadd__(self, other) -> PlainQuantity[MagnitudeT]:
-        ...
+    def __iadd__(self, other) -> PlainQuantity[MagnitudeT]: ...
 
     def __iadd__(self, other):
         if isinstance(other, datetime.datetime):
@@ -1280,8 +1313,8 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
     def __abs__(self) -> PlainQuantity[MagnitudeT]:
         return self.__class__(abs(self._magnitude), self._units)
 
-    def __round__(self, ndigits: Optional[int] = 0) -> PlainQuantity[MagnitudeT]:
-        return self.__class__(round(self._magnitude, ndigits=ndigits), self._units)
+    def __round__(self, ndigits: int | None = None) -> PlainQuantity[int]:
+        return self.__class__(round(self._magnitude, ndigits), self._units)
 
     def __pos__(self) -> PlainQuantity[MagnitudeT]:
         return self.__class__(operator.pos(self._magnitude), self._units)
@@ -1449,6 +1482,11 @@ class PlainQuantity(Generic[MagnitudeT], PrettyIPython, SharedRegistryObject):
         """Return a list of the of non-multiplicative units of the PlainQuantity object."""
         return []
 
+    @property
+    def _is_logarithmic(self) -> bool:
+        """Check if the PlainQuantity object has logarithmic units."""
+        return False
+
     def _get_delta_units(self) -> list[str]:
         """Return list of delta units ot the PlainQuantity object."""
         return [u for u in self._units if u.startswith("delta_")]
