@@ -1,14 +1,16 @@
 """
-    pint.matplotlib
-    ~~~~~~~~~~~~~~~
+pint.matplotlib
+~~~~~~~~~~~~~~~
 
-    Functions and classes related to working with Matplotlib's support
-    for plotting with units.
+Functions and classes related to working with Matplotlib's support
+for plotting with units.
 
-    :copyright: 2017 by Pint Authors, see AUTHORS for more details.
-    :license: BSD, see LICENSE for more details.
+:copyright: 2017 by Pint Authors, see AUTHORS for more details.
+:license: BSD, see LICENSE for more details.
 """
 
+from __future__ import annotations
+
 import matplotlib.units
 
 from .util import iterable, sized
@@ -19,7 +21,8 @@ class PintAxisInfo(matplotlib.units.AxisInfo):
 
     def __init__(self, units):
         """Set the default label to the pretty-print of the unit."""
-        super().__init__(label="{:P}".format(units))
+        formatter = units._REGISTRY.mpl_formatter
+        super().__init__(label=formatter.format(units))
 
 
 class PintConverter(matplotlib.units.ConversionInterface):
@@ -30,20 +33,21 @@ class PintConverter(matplotlib.units.ConversionInterface):
         self._reg = registry
 
     def convert(self, value, unit, axis):
-        """Convert :`Quantity` instances for matplotlib to use.
-        """
-        if iterable(value):
-            return [self._convert_value(v, unit, axis) for v in value]
-        else:
-            return self._convert_value(value, unit, axis)
-
-    def _convert_value(self, value, unit, axis):
-        """Handle converting using attached unit or falling back to axis units.
-        """
+        """Convert :`Quantity` instances for matplotlib to use."""
+        # Short circuit for arrays
         if hasattr(value, "units"):
             return value.to(unit).magnitude
-        else:
-            return self._reg.Quantity(value, axis.get_units()).to(unit).magnitude
+        if iterable(value):
+            return [self._convert_value(v, unit, axis) for v in value]
+
+        return self._convert_value(value, unit, axis)
+
+    def _convert_value(self, value, unit, axis):
+        """Handle converting using attached unit or falling back to axis units."""
+        if hasattr(value, "units"):
+            return value.to(unit).magnitude
+
+        return self._reg.Quantity(value, axis.get_units()).to(unit).magnitude
 
     @staticmethod
     def axisinfo(unit, axis):
@@ -53,8 +57,7 @@ class PintConverter(matplotlib.units.ConversionInterface):
 
     @staticmethod
     def default_units(x, axis):
-        """Get the default unit to use for the given combination of unit and axis.
-        """
+        """Get the default unit to use for the given combination of unit and axis."""
         if iterable(x) and sized(x):
             return getattr(x[0], "units", None)
         return getattr(x, "units", None)
