@@ -1,11 +1,11 @@
 """
-    pint.util
-    ~~~~~~~~~
+pint.util
+~~~~~~~~~
 
-    Miscellaneous functions for pint.
+Miscellaneous functions for pint.
 
-    :copyright: 2016 by Pint Authors, see AUTHORS for more details.
-    :license: BSD, see LICENSE for more details.
+:copyright: 2016 by Pint Authors, see AUTHORS for more details.
+:license: BSD, see LICENSE for more details.
 """
 
 from __future__ import annotations
@@ -14,31 +14,28 @@ import logging
 import math
 import operator
 import re
-from collections.abc import Mapping, Iterable, Iterator
+import tokenize
+import types
+from collections import deque
+from collections.abc import Callable, Generator, Hashable, Iterable, Iterator, Mapping
 from fractions import Fraction
 from functools import lru_cache, partial
 from logging import NullHandler
 from numbers import Number
 from token import NAME, NUMBER
-import tokenize
-import types
 from typing import (
     TYPE_CHECKING,
-    ClassVar,
-    Callable,
-    TypeVar,
     Any,
-    Optional,
+    ClassVar,
+    TypeAlias,
+    TypeVar,
 )
-from collections.abc import Hashable, Generator
 
+from . import pint_eval
+from ._typing import Scalar
 from .compat import NUMERIC_TYPES, Self
 from .errors import DefinitionSyntaxError
-from .formatting import format_unit
 from .pint_eval import build_eval_tree
-from . import pint_eval
-
-from ._typing import Scalar
 
 if TYPE_CHECKING:
     from ._typing import QuantityOrUnitLike
@@ -52,11 +49,8 @@ T = TypeVar("T")
 TH = TypeVar("TH", bound=Hashable)
 TT = TypeVar("TT", bound=type)
 
-# TODO: Change when Python 3.10 becomes minimal version.
-# ItMatrix: TypeAlias = Iterable[Iterable[PintScalar]]
-# Matrix: TypeAlias = list[list[PintScalar]]
-ItMatrix = Iterable[Iterable[Scalar]]
-Matrix = list[list[Scalar]]
+ItMatrix: TypeAlias = Iterable[Iterable[Scalar]]
+Matrix: TypeAlias = list[list[Scalar]]
 
 
 def _noop(x: T) -> T:
@@ -65,8 +59,8 @@ def _noop(x: T) -> T:
 
 def matrix_to_string(
     matrix: ItMatrix,
-    row_headers: Optional[Iterable[str]] = None,
-    col_headers: Optional[Iterable[str]] = None,
+    row_headers: Iterable[str] | None = None,
+    col_headers: Iterable[str] | None = None,
     fmtfun: Callable[
         [
             Scalar,
@@ -181,9 +175,7 @@ def column_echelon_form(
             ItMatrix,
         ],
         Matrix,
-    ] = (
-        transpose if transpose_result else _noop
-    )
+    ] = transpose if transpose_result else _noop
 
     ech_matrix = matrix_apply(
         transpose(matrix),
@@ -233,7 +225,7 @@ def column_echelon_form(
     return _transpose(ech_matrix), _transpose(id_matrix), swapped
 
 
-def pi_theorem(quantities: dict[str, Any], registry: Optional[UnitRegistry] = None):
+def pi_theorem(quantities: dict[str, Any], registry: UnitRegistry | None = None):
     """Builds dimensionless quantities using the Buckingham π theorem
 
     Parameters
@@ -310,8 +302,8 @@ def pi_theorem(quantities: dict[str, Any], registry: Optional[UnitRegistry] = No
 
 
 def solve_dependencies(
-    dependencies: dict[TH, set[TH]]
-) -> Generator[set[TH], None, None]:
+    dependencies: dict[TH, set[TH]],
+) -> Generator[set[TH]]:
     """Solve a dependency graph.
 
     Parameters
@@ -348,9 +340,7 @@ def solve_dependencies(
         yield t
 
 
-def find_shortest_path(
-    graph: dict[TH, set[TH]], start: TH, end: TH, path: Optional[list[TH]] = None
-):
+def find_shortest_path(graph: dict[TH, set[TH]], start: TH, end: TH):
     """Find shortest path between two nodes within a graph.
 
     Parameters
@@ -362,37 +352,34 @@ def find_shortest_path(
         Starting node.
     end
         End node.
-    path
-        Path to prepend to the one found.
-        (default = None, empty path.)
 
     Returns
     -------
     list[TH]
         The shortest path between two nodes.
     """
-    path = (path or []) + [start]
+    path = [start]
     if start == end:
         return path
 
-    # TODO: raise ValueError when start not in graph
-    if start not in graph:
-        return None
+    fifo = deque()
+    fifo.append((start, path))
+    visited = set()
+    while fifo:
+        node, path = fifo.popleft()
+        visited.add(node)
+        for adjascent_node in graph[node] - visited:
+            if adjascent_node == end:
+                return path + [adjascent_node]
+            else:
+                fifo.append((adjascent_node, path + [adjascent_node]))
 
-    shortest = None
-    for node in graph[start]:
-        if node not in path:
-            newpath = find_shortest_path(graph, node, end, path)
-            if newpath:
-                if not shortest or len(newpath) < len(shortest):
-                    shortest = newpath
-
-    return shortest
+    return None
 
 
 def find_connected_nodes(
-    graph: dict[TH, set[TH]], start: TH, visited: Optional[set[TH]] = None
-) -> Optional[set[TH]]:
+    graph: dict[TH, set[TH]], start: TH, visited: set[TH] | None = None
+) -> set[TH] | None:
     """Find all nodes connected to a start node within a graph.
 
     Parameters
@@ -452,12 +439,12 @@ class UnitsContainer(Mapping[str, Scalar]):
     __slots__ = ("_d", "_hash", "_one", "_non_int_type")
 
     _d: udict
-    _hash: Optional[int]
+    _hash: int | None
     _one: Scalar
     _non_int_type: type
 
     def __init__(
-        self, *args: Any, non_int_type: Optional[type] = None, **kwargs: Any
+        self, *args: Any, non_int_type: type | None = None, **kwargs: Any
     ) -> None:
         if args and isinstance(args[0], UnitsContainer):
             default_non_int_type = args[0]._non_int_type
@@ -502,7 +489,7 @@ class UnitsContainer(Mapping[str, Scalar]):
         UnitsContainer
             A copy of this container.
         """
-        newval = self._d[key] + value
+        newval = self._d[key] + self._normalize_nonfloat_value(value)
         new = self.copy()
         if newval:
             new._d[key] = newval
@@ -550,6 +537,9 @@ class UnitsContainer(Mapping[str, Scalar]):
         new._hash = None
         return new
 
+    def unit_items(self) -> Iterable[tuple[str, Scalar]]:
+        return self._d.items()
+
     def __iter__(self) -> Iterator[str]:
         return iter(self._d)
 
@@ -606,9 +596,15 @@ class UnitsContainer(Mapping[str, Scalar]):
         return f"<UnitsContainer({tmp})>"
 
     def __format__(self, spec: str) -> str:
+        # TODO: provisional
+        from .formatting import format_unit
+
         return format_unit(self, spec)
 
     def format_babel(self, spec: str, registry=None, **kwspec) -> str:
+        # TODO: provisional
+        from .formatting import format_unit
+
         return format_unit(self, spec, registry=registry, **kwspec)
 
     def __copy__(self):
@@ -654,7 +650,7 @@ class UnitsContainer(Mapping[str, Scalar]):
 
         new = self.copy()
         for key, value in other.items():
-            new._d[key] -= value
+            new._d[key] -= self._normalize_nonfloat_value(value)
             if new._d[key] == 0:
                 del new._d[key]
 
@@ -668,6 +664,11 @@ class UnitsContainer(Mapping[str, Scalar]):
 
         return self**-1
 
+    def _normalize_nonfloat_value(self, value: Scalar) -> Scalar:
+        if not isinstance(value, int) and not isinstance(value, self._non_int_type):
+            return self._non_int_type(value)  # type: ignore[no-any-return]
+        return value
+
 
 class ParserHelper(UnitsContainer):
     """The ParserHelper stores in place the product of variables and
@@ -1000,24 +1001,29 @@ class PrettyIPython:
     default_format: str
 
     def _repr_html_(self) -> str:
-        if "~" in self.default_format:
+        if "~" in self._REGISTRY.formatter.default_format:
             return f"{self:~H}"
         return f"{self:H}"
 
     def _repr_latex_(self) -> str:
-        if "~" in self.default_format:
+        if "~" in self._REGISTRY.formatter.default_format:
             return f"${self:~L}$"
         return f"${self:L}$"
 
     def _repr_pretty_(self, p, cycle: bool):
-        if "~" in self.default_format:
+        # if cycle:
+        if "~" in self._REGISTRY.formatter.default_format:
             p.text(f"{self:~P}")
         else:
             p.text(f"{self:P}")
+        # else:
+        #     p.pretty(self.magnitude)
+        #     p.text(" ")
+        #     p.pretty(self.units)
 
 
 def to_units_container(
-    unit_like: QuantityOrUnitLike, registry: Optional[UnitRegistry] = None
+    unit_like: QuantityOrUnitLike, registry: UnitRegistry | None = None
 ) -> UnitsContainer:
     """Convert a unit compatible type to a UnitsContainer.
 
@@ -1054,7 +1060,7 @@ def to_units_container(
 
 
 def infer_base_unit(
-    unit_like: QuantityOrUnitLike, registry: Optional[UnitRegistry] = None
+    unit_like: QuantityOrUnitLike, registry: UnitRegistry | None = None
 ) -> UnitsContainer:
     """
     Given a Quantity or UnitLike, give the UnitsContainer for it's plain units.
